/* 
 * "Small Hello World" example.  aaa
 * 
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example 
 * designs. It requires a STDOUT  device in your system's hardware. 
 *
 * The purpose of this example is to demonstrate the smallest possible Hello 
 * World application, using the Nios II HAL library.  The memory footprint
 * of this hosted application is ~332 bytes by default using the standard 
 * reference design.  For a more fully featured Hello World application
 * example, see the example titled "Hello World".
 *
 * The memory footprint of this example has been reduced by making the
 * following changes to the normal "Hello World" example.
 * Check in the Nios II Software Developers Manual for a more complete 
 * description.
 * 
 * In the SW Application project (small_hello_world):
 *
 *  - In the C/C++ Build page
 * 
 *    - Set the Optimization Level to -Os
 * 
 * In System Library project (small_hello_world_syslib):
 *  - In the C/C++ Build page
 * 
 *    - Set the Optimization Level to -Os
 * 
 *    - Define the preprocessor option ALT_NO_INSTRUCTION_EMULATION 
 *      This removes software exception handling, which means that you cannot 
 *      run code compiled for Nios II cpu with a hardware multiplier on a core 
 *      without a the multiply unit. Check the Nios II Software Developers 
 *      Manual for more details.
 *
 *  - In the System Library page:
 *    - Set Periodic system timer and Timestamp timer to none
 *      This prevents the automatic inclusion of the timer driver.
 *
 *    - Set Max file descriptors to 4
 *      This reduces the size of the file handle pool.
 *
 *    - Check Main function does not exit
 *    - Uncheck Clean exit (flush buffers)
 *      This removes the unneeded call to exit when main returns, since it
 *      won't.
 *
 *    - Check Don't use C++
 *      This builds without the C++ support code.
 *
 *    - Check Small C library
 *      This uses a reduced functionality C library, which lacks  
 *      support for buffering, file IO, floating point and getch(), etc. 
 *      Check the Nios II Software Developers Manual for a complete list.
 *
 *    - Check Reduced device drivers
 *      This uses reduced functionality drivers if they're available. For the
 *      standard design this means you get polled UART and JTAG UART drivers,
 *      no support for the LCD driver and you lose the ability to program 
 *      CFI compliant flash devices.
 *
 *    - Check Access device drivers directly
 *      This bypasses the device file system to access device drivers directly.
 *      This eliminates the space required for the device file system services.
 *      It also provides a HAL version of libc services that access the drivers
 *      directly, further reducing space. Only a limited number of libc
 *      functions are available in this configuration.
 *
 *    - Use ALT versions of stdio routines:
 *
 *           Function                  Description
 *        ===============  =====================================
 *        alt_printf       Only supports %s, %x, and %c ( < 1 Kbyte)
 *        alt_putstr       Smaller overhead than puts with direct drivers
 *                         Note this function doesn't add a newline.
 *        alt_putchar      Smaller overhead than putchar with direct drivers
 *        alt_getchar      Smaller overhead than getchar with direct drivers
 *
 */

//@LAP\tgi荞݁j@VZOLEDn[hEFA

#include "sys/alt_stdio.h"
#include "system.h"
#include "sys/alt_irq.h"
#include "user_blk_a_regs.h"

volatile int edge_capture_sw;
volatile char sw_on;
volatile int dev_reg;
// time
//volatile int time_10us;
//volatile int time_100us;
//volatile int time_1ms;
//volatile int time_10ms;
//volatile int time_100ms;
//volatile int time_1s;
//volatile int time_10s;
//荞݃nh

static void handle_sw_interrupts(void* context, alt_u32 id)
{
  volatile int* edge_capture_sw_ptr = (volatile int*) context;
  sw_on = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);//foCXlǂݏo
  dev_reg = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);//foCXlǂݏo;
  //IOWR_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16, sw_on);//foCX֏
  //time_10us = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+32);
  //time_100us = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+36);
  //time_1ms = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+40);
  //time_10ms = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+44);
  //time_100ms = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+48);
  //time_1s = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+52);
  //time_10s = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+56);
  IOWR_USER_BLK_A_EDGE_CAP(USER_BLK_A_0_BASE, 0);//荞݃NA
}


static void init_sw()
{
  void* edge_capture_sw_ptr = (void*) &edge_capture_sw;
  IOWR_USER_BLK_A_IRQ_MASK(USER_BLK_A_0_BASE, 0xf);//@荞݂Lɂ
  IOWR_USER_BLK_A_EDGE_CAP(USER_BLK_A_0_BASE, 0x0);//@荞݂NA
  sw_on = 0;
  //荞݃nho^
  alt_irq_register( USER_BLK_A_0_IRQ, edge_capture_sw_ptr, handle_sw_interrupts ); 
}


//\
/*
void serial_ctrl()
{ 
  if (sw_on == 0x1)
    alt_printf("start \n");
//  if (sw_on == 0xd)
//    alt_printf("lap  %x%x%x%x S \n",time_10ms,time_1ms,time_100us,time_10us);
  if (sw_on == 0x2)
    alt_printf("stop %x.%x%x S \n",time_1s,time_100ms,time_10ms); 
  if (sw_on == 0x4)
    alt_printf("lap %x.%x%x S \n",time_1s,time_100ms,time_10ms);
//  if (sw_on == 0x7) //
//    alt_printf("clear \n");
  //IOWR_USER_BLK_A_DATA(USER_BLK_A_0_BASE,0);
}
*/
//\

int main()
{ 
  //int dev_reg;
  //int sw_on_flg;
  //char sw_on;
  int time_int;
  int time_int_b;
  int time_10ms;
  int time_100ms;
  int time_1s;
  int time_10ms_b;
  int time_100ms_b;
  int time_1s_b;
  int lap_index;
  int lap_index_max;
  int count_exec;
  int lap_time[16];
  init_sw();//
  alt_putstr("reset\n");
  lap_index = 0;
  count_exec=0;
  //sw_on_flg = 0xff & IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE);//foCXlǂݏo;
  /* Event loop never exits. */
  while (1){
    //dev_reg = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);//foCXlǂݏo;
    //sw_on = 0xff & dev_reg;//foCXlǂݏo
    //sw_on = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);
    //if(sw_on_flg != sw_on) {
    if(sw_on !=  0) {
      //sw_on = dev_reg;
      //sw_on_flg = sw_on;
      //dev_reg = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);//foCXlǂݏo;
      time_10ms = 0xff & (dev_reg >> 8) ;
      time_100ms = 0xff & (dev_reg >> 16) ;
      time_1s = 0xff & (dev_reg >> 24);
      time_int = time_10ms + (time_100ms * 10) + (time_1s *100);
      IOWR_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16,0);
      if (sw_on == 0x1){  //start
        lap_index = 0;
        time_int_b =0;
        time_10ms_b =  0;
        time_100ms_b = 0;
        time_1s_b =    0;
        count_exec =1;
        alt_printf("start \n");
        }
      if (sw_on == 0x2){//stop
        //lap_index = 0;
        count_exec =0;        
        alt_printf("stop  %x.%x%x S \n",time_1s,time_100ms,time_10ms);
        //alt_printf("stop  %d\n",time_1);
        time_int_b =0;
        time_10ms_b =  0;
        time_100ms_b = 0; 
        time_1s_b =    0;
        lap_index_max = lap_index; 
        for(lap_index=0;lap_index < lap_index_max;lap_index++){
          time_int = lap_time[lap_index] ;
          time_1s = time_int/100;
          time_100ms = (time_int -time_1s*100)/10;
          time_10ms =  (time_int - time_1s*100 - time_100ms*10);
          time_int_b =  time_int - time_int_b ;
          time_1s_b = time_int_b/100;
          time_100ms_b = (time_int_b - time_1s_b*100)/10;
          time_10ms_b =  (time_int_b - time_1s_b*100 -time_100ms_b*10);
          alt_printf("lap%x %x%x%x uS  diff %x%x%x uS \n",(lap_index+1),time_1s,time_100ms,time_10ms,time_1s_b,time_100ms_b,time_10ms_b); 
          time_int_b =  time_int ;
        }
      }
      if (sw_on == 0x4){  //lap
        if (count_exec ==1){
          lap_time[lap_index] = time_int;
          lap_index++;
        }/*
        else  { 
          time_int = lap_time[lap_index] ;
          time_1s = time_int/100;
          time_100ms = (time_int -time_1s*100)/10;
          time_10ms =  (time_int - time_1s*100 - time_100ms*10);}
        //alt_printf("lap  %d\n",time_1);
        //alt_printf("test  %x.%x %x S \n",time_1s_b,time_100ms_b,time_10ms_b);
        //alt_printf("test1  %x.%x %x S \n",time_1s,time_100ms,time_10ms);
        //alt_printf("test2  %x.%x %x S \n",time_a_1s,time_a_100ms,time_a_10ms);
        time_int_b =  time_int - time_int_b ;
        time_1s_b = time_int_b/100;
        time_100ms_b = (time_int_b - time_1s_b*100)/10;
        time_10ms_b =  (time_int_b - time_1s_b*100 -time_100ms_b*10);
        alt_printf("lap%x %x.%x %x S  diff %x.%x %x S\n",lap_index,time_1s,time_100ms,time_10ms,time_1s_b,time_100ms_b,time_10ms_b); 
        time_int_b =  time_int ;
        lap_index++;} */
      }
    sw_on = 0 ;  
    }
  }
  return 0;
}
