/* 
 * "Small Hello World" example.  aaa
 * 
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example 
 * designs. It requires a STDOUT  device in your system's hardware. 
 *
 * The purpose of this example is to demonstrate the smallest possible Hello 
 * World application, using the Nios II HAL library.  The memory footprint
 * of this hosted application is ~332 bytes by default using the standard 
 * reference design.  For a more fully featured Hello World application
 * example, see the example titled "Hello World".
 *
 * The memory footprint of this example has been reduced by making the
 * following changes to the normal "Hello World" example.
 * Check in the Nios II Software Developers Manual for a more complete 
 * description.
 * 
 * In the SW Application project (small_hello_world):
 *
 *  - In the C/C++ Build page
 * 
 *    - Set the Optimization Level to -Os
 * 
 * In System Library project (small_hello_world_syslib):
 *  - In the C/C++ Build page
 * 
 *    - Set the Optimization Level to -Os
 * 
 *    - Define the preprocessor option ALT_NO_INSTRUCTION_EMULATION 
 *      This removes software exception handling, which means that you cannot 
 *      run code compiled for Nios II cpu with a hardware multiplier on a core 
 *      without a the multiply unit. Check the Nios II Software Developers 
 *      Manual for more details.
 *
 *  - In the System Library page:
 *    - Set Periodic system timer and Timestamp timer to none
 *      This prevents the automatic inclusion of the timer driver.
 *
 *    - Set Max file descriptors to 4
 *      This reduces the size of the file handle pool.
 *
 *    - Check Main function does not exit
 *    - Uncheck Clean exit (flush buffers)
 *      This removes the unneeded call to exit when main returns, since it
 *      won't.
 *
 *    - Check Don't use C++
 *      This builds without the C++ support code.
 *
 *    - Check Small C library
 *      This uses a reduced functionality C library, which lacks  
 *      support for buffering, file IO, floating point and getch(), etc. 
 *      Check the Nios II Software Developers Manual for a complete list.
 *
 *    - Check Reduced device drivers
 *      This uses reduced functionality drivers if they're available. For the
 *      standard design this means you get polled UART and JTAG UART drivers,
 *      no support for the LCD driver and you lose the ability to program 
 *      CFI compliant flash devices.
 *
 *    - Check Access device drivers directly
 *      This bypasses the device file system to access device drivers directly.
 *      This eliminates the space required for the device file system services.
 *      It also provides a HAL version of libc services that access the drivers
 *      directly, further reducing space. Only a limited number of libc
 *      functions are available in this configuration.
 *
 *    - Use ALT versions of stdio routines:
 *
 *           Function                  Description
 *        ===============  =====================================
 *        alt_printf       Only supports %s, %x, and %c ( < 1 Kbyte)
 *        alt_putstr       Smaller overhead than puts with direct drivers
 *                         Note this function doesn't add a newline.
 *        alt_putchar      Smaller overhead than putchar with direct drivers
 *        alt_getchar      Smaller overhead than getchar with direct drivers
 *
 */

//@LAP\tgi|[Oj@VZOLEDn[h

#include "sys/alt_stdio.h"
#include "system.h"
#include "user_blk_a_regs.h"
int main()
{ 
  int dev_reg;
  char sw_on;
  int time_int;
  int time_int_b;
  int time_10ms;
  int time_100ms;
  int time_1s;
  int time_10ms_b;
  int time_100ms_b;
  int time_1s_b;
  int lap_index;
  int lap_index_max;
  int count_exec;
  int lap_time[16];
  alt_putstr("reset\n");
  lap_index = 0;
  count_exec=0;
  while (1){
    sw_on = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);
    if(sw_on !=  0) {
      dev_reg = IORD_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16);//foCXlǂݏo;
      time_10ms = 0xff & (dev_reg >> 8) ;
      time_100ms = 0xff & (dev_reg >> 16) ;
      time_1s = 0xff & (dev_reg >> 24);
      time_int = time_10ms + (time_100ms * 10) + (time_1s *100);
      IOWR_USER_BLK_A_DATA(USER_BLK_A_0_BASE+16,0);
      if (sw_on == 0x1){  //start
        lap_index = 0;
        time_int_b =0;
        time_10ms_b =  0;
        time_100ms_b = 0;
        time_1s_b =    0;
        count_exec =1;
        alt_printf("start \n");
        }
      if (sw_on == 0x2){//stop
        count_exec =0;        
        alt_printf("stop  %x.%x%x S \n",time_1s,time_100ms,time_10ms);
        time_int_b =0;
        time_10ms_b =  0;
        time_100ms_b = 0; 
        time_1s_b =    0;
        lap_index_max = lap_index; 
        for(lap_index=0;lap_index < lap_index_max;lap_index++){
          time_int = lap_time[lap_index] ;
          time_1s = time_int/100;
          time_100ms = (time_int -time_1s*100)/10;
          time_10ms =  (time_int - time_1s*100 - time_100ms*10);
          time_int_b =  time_int - time_int_b ;
          time_1s_b = time_int_b/100;
          time_100ms_b = (time_int_b - time_1s_b*100)/10;
          time_10ms_b =  (time_int_b - time_1s_b*100 -time_100ms_b*10);
          alt_printf("lap%x %x%x%x uS  diff %x%x%x uS \n",lap_index,time_1s,time_100ms,time_10ms,time_1s_b,time_100ms_b,time_10ms_b); 
          time_int_b =  time_int ;
        }
      }
      if (sw_on == 0x4){  //lap@lۑ
        if (count_exec ==1){
          lap_time[lap_index] = time_int;
          lap_index++;
        }
      }
    }
  }
  return 0;
}
